# docTool module

Usage:

     node streamline-util/lib/docTool [path]

Extracts documentation comments from `.js` files and generates `API.md` file 
under package root.

Top of source file must contain `/// !doc` marker to enable doc extraction.  
Documentation comments must start with `/// ` (with 1 trailing space).  
Extraction can be turned off with `/// !nodoc` and turned back on with `/// !doc`.

The tool can also be invoked programatically with:

* `doc = docTool.generate(_, path)`
  extracts documentation comments from file `path`

# Flows Module

The `streamline-util/lib/flows` module contains some handy utilities for streamline code

## Array utilities

The following functions are async equivalents of the ES5 Array methods (`forEach`, `map`, `filter`, ...)

* `flows.each(_, array, fn, [thisObj])`  
  applies `fn` sequentially to the elements of `array`.  
  `fn` is called as `fn(_, elt, i)`.
* `result = flows.map(_, array, fn, [thisObj])`  
  transforms `array` by applying `fn` to each element in turn.  
  `fn` is called as `fn(_, elt, i)`.
* `result = flows.filter(_, array, fn, [thisObj])`  
  generates a new array that only contains the elements that satisfy the `fn` predicate.  
  `fn` is called as `fn(_, elt)`.
* `bool = flows.every(_, array, fn, [thisObj])`  
  returns true if `fn` is true on every element (if `array` is empty too).  
  `fn` is called as `fn(_, elt)`.
* `bool = flows.some(_, array, fn, [thisObj])`  
  returns true if `fn` is true for at least one element.  
  `fn` is called as `fn(_, elt)`.
* `result = flows.reduce(_, array, fn, val, [thisObj])`  
  reduces by applying `fn` to each element.  
  `fn` is called as `val = fn(_, val, elt, i, array)`.
* `result = flows.reduceRight(_, array, fn, val, [thisObj])`  
  reduces from end to start by applying `fn` to each element.  
  `fn` is called as `val = fn(_, val, elt, i, array)`.

## Object utility

The following function can be used to iterate through object properties:

* `flows.eachKey(_, obj, fn)`  
  calls `fn(_, key, obj[key])` for every `key` in `obj`.

## Workflow Utilities

* `fun = flows.funnel(max)`  
  limits the number of concurrent executions of a given code block.

The `funnel` function is typically used with the following pattern:

    // somewhere
    var myFunnel = flows.funnel(10); // create a funnel that only allows 10 concurrent executions.
    
    // elsewhere
    myFunnel(_, function(_) { /* code with at most 10 concurrent executions */ });

The `diskUsage2.js` example demonstrates how these calls can be combined to control concurrent execution.

The `funnel` function can also be used to implement critical sections. Just set funnel's `max` parameter to 1.

* `results = flows.collect(_, futures)`  
  collects the results of an array of futures

## Context propagation

Streamline also allows you to propagate a global context along a chain of calls and callbacks.
This context can be used like TLS (Thread Local Storage) in a threaded environment.
It allows you to have several active chains that each have their own global context.

This kind of context is very handy to store information that all calls should be able to access
but that you don't want to pass explicitly via function parameters. The most obvious example is
the `locale` that each request may set differently and that your low level libraries should
be able to retrieve to format messages.

The `streamline.flows` module exposes two functions to manipulate the context:

* `oldCtx = flows.setContext(ctx)`  
  sets the context (and returns the old context).
* `ctx = flows.getContext()`  
  returns the current context.
